@@ -29,6 +29,7 @@ from glue.core.message import (DataCollectionAddMessage,
                                SubsetDeleteMessage)
 from glue.core.state_objects import State
 from glue.core.subset import Subset, RangeSubsetState, RoiSubsetState
+from glue_astronomy.spectral_coordinates import SpectralCoordinates
 from glue_jupyter.app import JupyterApplication
 from glue_jupyter.common.toolbar_vuetify import read_icon
 from glue_jupyter.state_traitlets_helpers import GlueState
@@ -402,6 +403,26 @@ class Application(VuetifyTemplate, HubListener):
         ref_data = dc[reference_data] if reference_data else dc[0]
         linked_data = dc[data_to_be_linked] if data_to_be_linked else dc[-1]
 
+        if linked_data.meta.get('Plugin', None) == 'SpectralExtraction':
+            if 'Trace' in linked_data.meta:
+                links = [LinkSame(linked_data.components[2], ref_data.components[0]),
+                         LinkSame(linked_data.components[0], ref_data.components[1])]
+                dc.add_link(links)
+                return
+            else:
+                links = [LinkSame(linked_data.components[0], ref_data.components[0]),
+                         LinkSame(linked_data.components[1], ref_data.components[1])]
+                dc.add_link(links)
+                return
+
+        # The glue-astronomy SpectralCoordinates currently seems incompatible with glue
+        # WCSLink. This gets around it until there's an upstream fix.
+        if isinstance(linked_data.coords, SpectralCoordinates):
+            wc_old = ref_data.world_component_ids[-1]
+            wc_new = linked_data.world_component_ids[0]
+            self.data_collection.add_link(LinkSame(wc_old, wc_new))
+            return
+
         try:
             if linked_data.meta.get("Plugin", None) == 'GaussianSmooth':
                 raise AttributeError
@@ -604,8 +625,8 @@ class Application(VuetifyTemplate, HubListener):
         for layer_state in viewer.state.layers:
             label = layer_state.layer.label
 
-            if hasattr(layer_state, 'layer') and \
-                    (data_label is None or label == data_label):
+            if (hasattr(layer_state, 'layer') and
+                    (data_label is None or label == data_label)):
 
                 # For raw data, just include the data itself
                 if isinstance(layer_state.layer, BaseData):
@@ -614,7 +635,9 @@ class Application(VuetifyTemplate, HubListener):
                     if cls is not None:
                         # If data is one-dimensional, assume that it can be
                         #  collapsed via the defined statistic
-                        if cls == Spectrum1D:
+                        if 'Trace' in layer_data.meta:
+                            layer_data = layer_data.get_object()
+                        elif cls == Spectrum1D:
                             layer_data = layer_data.get_object(cls=cls,
                                                                statistic=statistic)
                         else:
@@ -1156,7 +1179,7 @@ class Application(VuetifyTemplate, HubListener):
         # resize tray items
         for tray_item in self.state.tray_items:
             # access the actual plugin object (there is no store for plugins)
-            tray_obj = self.widgets.get(tray_item['widget'].split('IPY_MODEL_')[1])
+            tray_obj = widget_serialization['from_json'](tray_item['widget'], None)
             for bqplot_fig in tray_obj.bqplot_figs_resize:
                 bqplot_fig.layout.height = '99.9%'
                 bqplot_fig.layout.height = '100%'
@@ -1229,8 +1252,13 @@ class Application(VuetifyTemplate, HubListener):
 
         replace = event.get('replace', False)
         selected_items = viewer_item['selected_data_items']
+
         if replace and visible:
-            selected_items = {id: 'visible' if id == item_id else 'hidden' for id in selected_items.keys()}  # noqa
+            # trace items remain selected, even if the mode is in replace
+            keep_selected = [k for k, v in viewer_item['selected_data_items'].items()
+                             if v != 'hidden' and
+                             self._get_data_item_by_id(k).get('type') == 'trace']
+            selected_items = {id: 'visible' if id == item_id or id in keep_selected else 'hidden' for id in selected_items.keys()}  # noqa
         else:
             selected_items[item_id] = 'visible' if visible else 'hidden'
 
@@ -1252,6 +1280,10 @@ class Application(VuetifyTemplate, HubListener):
         kwargs = event.get('kwargs', {})
         return getattr(self._viewer_store[viewer_id], method)(*args, **kwargs)
 
+    def _get_data_item_by_id(self, data_id):
+        return next((x for x in self.state.data_items
+                     if x['id'] == data_id), None)
+
     def _update_selected_data_items(self, viewer_id, selected_items):
         # Find the active viewer
         viewer_item = self._viewer_item_by_id(viewer_id)
@@ -1266,8 +1298,7 @@ class Application(VuetifyTemplate, HubListener):
 
         # Include any selected data in the viewer
         for data_id, visibility in selected_items.items():
-            label = next((x['name'] for x in self.state.data_items
-                          if x['id'] == data_id), None)
+            label = self._get_data_item_by_id(data_id)['name']
 
             if label is None:
                 warnings.warn(f"No data item with id '{data_id}' found in "
@@ -1330,7 +1361,9 @@ class Application(VuetifyTemplate, HubListener):
             The Glue data collection add message containing information about
             the new data.
         """
-        self._link_new_data()
+        # We don't need to link the first data to itself
+        if len(self.data_collection) > 1:
+            self._link_new_data()
         data_item = self._create_data_item(msg.data)
         self.state.data_items.append(data_item)
 
@@ -1359,6 +1392,8 @@ class Application(VuetifyTemplate, HubListener):
             component_ids = [str(c) for c in data.component_ids()]
         if data.label == 'MOS Table':
             typ = 'table'
+        elif 'Trace' in data.meta:
+            typ = 'trace'
         elif ndims == 1:
             typ = '1d spectrum'
         elif ndims == 2 and wcsaxes is not None:
