@@ -152,6 +152,14 @@ class LineListTool(PluginTemplateMixin):
         if viewer_data is None:
             return
 
+        if viewer_data.spectral_axis.unit == u.pix:
+            # disable the plugin until we can address this properly (either using the wavelength
+            # solution to support pixels in line-lists, or properly displaying the extracted
+            # 1d spectrum in wavelength-space)
+            self.disabled_msg = 'Line Lists unavailable when x-axis is in pixels'
+        else:
+            self.disabled_msg = ''
+
         self._units["x"] = str(viewer_data.spectral_axis.unit)
         self._units["y"] = str(viewer_data.flux.unit)
 
@@ -813,7 +821,7 @@ class LineListTool(PluginTemplateMixin):
         if erase:
             try:
                 self._viewer.erase_spectral_lines(name_rest=name_rest)
-                del(self.line_mark_dict[name_rest])
+                del self.line_mark_dict[name_rest]
             except KeyError:
                 raise KeyError("line marks: {}".format(self._viewer.figure.marks))
         else:
