@@ -224,12 +224,11 @@ def test_subset_default_thickness(specviz_helper, spectrum1d):
     specviz_helper.load_spectrum(spectrum1d)
 
     sv = specviz_helper.app.get_viewer('spectrum-viewer')
-    tool = sv.toolbar.tools['bqplot:xrange']
-    tool.activate()
-    tool.interact.brushing = True
-    tool.interact.selected = [2.5, 3.5]
-    tool.interact.brushing = False
-
+    sv.toolbar.active_tool = sv.toolbar.tools['bqplot:xrange']
+    from glue.core.roi import XRangeROI
+    sv.apply_roi(XRangeROI(2.5, 3.5))
+    # _on_layers_update is not triggered within CI
+    sv._on_layers_update()
     assert sv.state.layers[-1].linewidth == 3
 
 
@@ -265,10 +264,10 @@ def test_plot_uncertainties(specviz_helper, spectrum1d):
 
     assert len([m for m in specviz_viewer.figure.marks if isinstance(m, LineUncertainties)]) == 0
 
-    specviz_viewer.show_uncertainties()
+    specviz_viewer.state.show_uncertainty = True
 
     assert len([m for m in specviz_viewer.figure.marks if isinstance(m, LineUncertainties)]) == 1
 
-    specviz_viewer.clean()
+    specviz_viewer.state.show_uncertainty = False
 
     assert len([m for m in specviz_viewer.figure.marks if isinstance(m, LineUncertainties)]) == 0
