@@ -1,8 +1,10 @@
 import numpy as np
 
+from jdaviz.configs.specviz import Specviz
 from jdaviz.core.helpers import ConfigHelper
 from jdaviz.core.events import SnackbarMessage
 from jdaviz.configs.default.plugins.line_lists.line_list_mixin import LineListMixin
+from jdaviz.configs.specviz2d.plugins.spectral_extraction.spectral_extraction import SpectralExtraction  # noqa
 
 __all__ = ['Specviz2d']
 
@@ -21,6 +23,16 @@ class Specviz2d(ConfigHelper, LineListMixin):
         spec2d = self.app.get_viewer("spectrum-2d-viewer")
         spec2d.scales['x'].observe(self._update_spec1d_x_axis)
 
+    @property
+    def specviz(self):
+        """
+        A Specviz helper (`~jdaviz.configs.specviz.helper.Specviz`) for the Jdaviz
+        application that is wrapped by Specviz2d.
+        """
+        if not hasattr(self, '_specviz'):
+            self._specviz = Specviz(app=self.app)
+        return self._specviz
+
     def _extend_world(self, spec1d, ext):
         # Extend 1D spectrum world axis to enable panning (within reason) past
         # the bounds of data
@@ -140,17 +152,28 @@ class Specviz2d(ConfigHelper, LineListMixin):
 
             # Collapse the 2D spectrum to 1D if no 1D spectrum provided
             if spectrum_1d is None:
-                self.app.load_data(spectrum_2d,
-                                   parser_reference="spec2d-1d-parser",
-                                   data_label=spectrum_1d_label,
-                                   show_in_viewer=show_in_viewer)
-
-                # Warn that this shouldn't be used for science
-                msg = ("The collapsed 1D spectrum is for quicklook"
-                       " purposes only. A robust extraction should be used for"
-                       " scientific use cases.")
-                msg = SnackbarMessage(msg, color='warning', sender=self,
-                                      timeout=10000)
+                # create a non-interactive (so that it does not create duplicate marks with the
+                # plugin-instance created later) instance of the SpectralExtraction plugin,
+                # and use the defaults to generate the initial 1D extracted spectrum
+                spext = SpectralExtraction(app=self.app, interactive=False)
+                # for some reason, the trailets are resetting to their default values even
+                # though _trace_dataset_selected was called internally to set them to reasonable
+                # new defaults.  We'll just call it again manually.
+                spext._trace_dataset_selected()
+                try:
+                    spext.export_extract_spectrum(add_data=True)
+                except Exception:
+                    msg = SnackbarMessage(
+                        "Automatic spectrum extraction failed.  See the spectral extraction"
+                        " plugin to perform a custom extraction",
+                        color='error', sender=self, timeout=10000)
+                else:
+                    # Warn that this shouldn't be used for science
+                    msg = SnackbarMessage(
+                        "The extracted 1D spectrum was generated automatically."
+                        " See the spectral extraction plugin for details or to"
+                        " perform a custom extraction.",
+                        color='warning', sender=self, timeout=10000)
                 self.app.hub.broadcast(msg)
 
         else:
