@@ -3,7 +3,7 @@ from glue.core.message import Message
 __all__ = ['NewViewerMessage', 'ViewerAddedMessage', 'ViewerRemovedMessage', 'LoadDataMessage',
            'AddDataMessage', 'SnackbarMessage', 'RemoveDataMessage',
            'AddLineListMessage', 'RowLockMessage',
-           'SliceSelectWavelengthMessage', 'SliceSelectSliceMessage',
+           'SliceSelectSliceMessage',
            'SliceToolStateMessage',
            'TableClickMessage']
 
@@ -256,17 +256,6 @@ class TableClickMessage(Message):
         return self._shared_image
 
 
-class SliceSelectWavelengthMessage(Message):
-    '''Message generated by the select slice plot plugin which is processed by the cubeviz helper'''
-    def __init__(self, wavelength=None, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-        self._wavelength = wavelength
-
-    @property
-    def wavelength(self):
-        return self._wavelength
-
-
 class SliceSelectSliceMessage(Message):
     '''Message generated by the cubeviz helper and processed by the slice plugin to sync
     slice selection across all viewers'''
