@@ -86,7 +86,7 @@ class TemplateMixin(VuetifyTemplate, HubListener):
 
 class PluginTemplateMixin(TemplateMixin):
     """
-    This base class can be inherited by all sidebar plugins to expose common functionality.
+    This base class can be inherited by all sidebar/tray plugins to expose common functionality.
     """
     disabled_msg = Unicode("").tag(sync=True)
     plugin_opened = Bool(False).tag(sync=True)
@@ -101,6 +101,16 @@ class PluginTemplateMixin(TemplateMixin):
         tray_names_open = [app_state.tray_items[i]['name'] for i in app_state.tray_items_open]
         self.plugin_opened = app_state.drawer and self._registry_name in tray_names_open
 
+    def open_in_tray(self):
+        """
+        Open the plugin in the sidebar/tray (and open the sidebar if it is not already).
+        """
+        app_state = self.app.state
+        app_state.drawer = True
+        index = [ti['name'] for ti in app_state.tray_items].index(self._registry_name)
+        if index not in app_state.tray_items_open:
+            app_state.tray_items_open = app_state.tray_items_open + [index]
+
 
 class BasePluginComponent(HubListener):
     """
@@ -722,6 +732,12 @@ class SubsetSelect(BaseSelectPluginComponent):
             if match is not None:
                 return match
 
+    @property
+    def selected_subset_state(self):
+        subset_group = [s for s in self.app.data_collection.subset_groups if
+                        s.label == self.selected][0]
+        return subset_group.subset_state
+
     def selected_min_max(self, spectrum1d):
         if self.selected_obj is None:
             return np.nanmin(spectrum1d.spectral_axis), np.nanmax(spectrum1d.spectral_axis)
@@ -925,6 +941,9 @@ class ViewerSelect(BaseSelectPluginComponent):
         def is_spectrum_viewer(viewer):
             return 'ProfileView' in viewer.__class__.__name__
 
+        def is_spectrum_2d_viewer(viewer):
+            return 'Profile2DView' in viewer.__class__.__name__
+
         def is_image_viewer(viewer):
             return 'ImageView' in viewer.__class__.__name__
 
@@ -1010,8 +1029,27 @@ class DatasetSelect(BaseSelectPluginComponent):
     """
     def __init__(self, plugin, items, selected,
                  filters=['not_from_plugin_model_fitting', 'layer_in_viewers'],
+                 default_text=None, manual_options=[],
                  default_mode='first'):
+        """
+        Parameters
+        ----------
+        plugin
+            the parent plugin object
+        items : str
+            the name of the items traitlet defined in ``plugin``
+        selected : str
+            the name of the selected traitlet defined in ``plugin``
+        default_text : str or None
+            the text to show for no selection.  If not provided or None, no entry will be provided
+            in the dropdown for no selection.
+        manual_options: list
+            list of options to provide that are not automatically populated by subsets.  If
+            ``default`` text is provided but not in ``manual_options`` it will still be included as
+            the first item in the list.
+        """
         super().__init__(plugin, items=items, selected=selected, filters=filters,
+                         default_text=default_text, manual_options=manual_options,
                          default_mode=default_mode)
         self._cached_properties += ["selected_dc_item"]
         # Add/Remove Data are triggered when checked/unchecked from viewers
@@ -1083,10 +1121,22 @@ class DatasetSelect(BaseSelectPluginComponent):
 
         def layer_in_spectrum_viewer(data):
             if not len(self.app.get_viewer_reference_names()):
-                # then this is a bar Application object, so ignore this filter
+                # then this is a bare Application object, so ignore this filter
                 return True
             return data.label in [l.layer.label for l in self.spectrum_viewer.layers] # noqa E741
 
+        def layer_in_spectrum_2d_viewer(data):
+            if not len(self.app.get_viewer_reference_names()):
+                # then this is a bare Application object, so ignore this filter
+                return True
+            return data.label in [l.layer.label for l in self.app.get_viewer('spectrum-2d-viewer').layers]  # noqa
+
+        def is_trace(data):
+            return hasattr(data, 'meta') and 'Trace' in data.meta
+
+        def not_trace(data):
+            return not is_trace(data)
+
         def is_image(data):
             return len(data.shape) == 3
 
@@ -1102,8 +1152,9 @@ class DatasetSelect(BaseSelectPluginComponent):
 
             return d
 
-        self.items = [_dc_to_dict(data) for data in self.app.data_collection
-                      if self._is_valid_item(data)]
+        manual_items = [{'label': label} for label in self.manual_options]
+        self.items = manual_items + [_dc_to_dict(data) for data in self.app.data_collection
+                                     if self._is_valid_item(data)]
         self._apply_default_selection()
         # future improvement: only clear cache if the selected data entry was changed?
         self._clear_cache(*self._cached_properties)
@@ -1256,7 +1307,8 @@ class AddResults(BasePluginComponent):
     """
     def __init__(self, plugin, label, label_default, label_auto,
                  label_invalid_msg, label_overwrite,
-                 add_to_viewer_items, add_to_viewer_selected):
+                 add_to_viewer_items, add_to_viewer_selected,
+                 label_whitelist_overwrite=[]):
         super().__init__(plugin, label=label,
                          label_default=label_default, label_auto=label_auto,
                          label_invalid_msg=label_invalid_msg, label_overwrite=label_overwrite,
@@ -1269,6 +1321,9 @@ class AddResults(BasePluginComponent):
         self.hub.subscribe(self, DataCollectionDeleteMessage,
                            handler=lambda _: self._on_label_changed())
 
+        # allows overwriting specific data entries not from the same plugin
+        self.label_whitelist_overwrite = label_whitelist_overwrite
+
         self.viewer = ViewerSelect(plugin, add_to_viewer_items, add_to_viewer_selected,
                                    manual_options=['None'],
                                    default_mode=self._handle_default_viewer_selected)
@@ -1292,7 +1347,8 @@ class AddResults(BasePluginComponent):
 
         for data in self.app.data_collection:
             if self.label == data.label:
-                if data.meta.get('Plugin', None) == self._plugin.__class__.__name__:
+                if data.meta.get('Plugin', None) == self._plugin.__class__.__name__ or\
+                        data.label in self.label_whitelist_overwrite:
                     self.label_invalid_msg = ''
                     self.label_overwrite = True
                     return
@@ -1304,39 +1360,59 @@ class AddResults(BasePluginComponent):
         self.label_invalid_msg = ''
         self.label_overwrite = False
 
-    def add_results_from_plugin(self, data_item):
+    def add_results_from_plugin(self, data_item, replace=None):
         """
         Add ``data_item`` to the app's data_collection according to the default or user-provided
         label and adds to any requested viewers.
         """
         if self.label_invalid_msg:
             raise ValueError(self.label_invalid_msg)
+
+        if replace is None:
+            replace = self.viewer.selected_reference != 'spectrum-viewer'
+
+        if self.label_overwrite and len(self.add_to_viewer_items) <= 2:
+            # the switch for add_to_viewer is hidden, and so the loaded state of the overwritten
+            # entry should be the same as the original entry (to avoid deleting reference data)
+            viewer_reference = self.add_to_viewer_items[-1]['reference']
+            viewer_item = self.app._viewer_item_by_reference(viewer_reference)
+            viewer = self.app.get_viewer(viewer_reference)
+            viewer_loaded_labels = [layer.layer.label for layer in viewer.layers]
+            add_to_viewer_selected = viewer_reference if self.label in viewer_loaded_labels else 'None'  # noqa
+            visible = self.label in viewer_item['visible_layers']
+        else:
+            add_to_viewer_selected = self.add_to_viewer_selected
+            visible = True
+
+        if self.label in self.app.data_collection:
+            if add_to_viewer_selected != 'None':
+                self.app.remove_data_from_viewer(self.viewer.selected_reference, self.label)
+            self.app.data_collection.remove(self.app.data_collection[self.label])
+
+        if not hasattr(data_item, 'meta'):
+            data_item.meta = {}
         data_item.meta['Plugin'] = self._plugin.__class__.__name__
         if self.app.config == 'mosviz':
             data_item.meta['mosviz_row'] = self.app.state.settings['mosviz_row']
-
-        replace = self.viewer.selected_reference != 'spectrum-viewer'
-
-        if self.label in self.app.data_collection:
-            self.app.remove_data_from_viewer(self.viewer.selected_reference, self.label)
-            self.app.data_collection.remove(self.app.data_collection[self.label])
-
         self.app.add_data(data_item, self.label)
 
-        if self.add_to_viewer_selected != 'None':
+        if add_to_viewer_selected != 'None':
             # replace the contents in the selected viewer with the results from this plugin
             # TODO: switch to an instance/classname check?
             self.app.add_data_to_viewer(self.viewer.selected_id,
                                         self.label, clear_other_data=False)
 
-            if replace:
+            if replace or not visible:
                 data_id = next((x['id'] for x in self.app.state.data_items
                                 if x['name'] == self.label), None)
                 self.app.vue_data_item_visibility({'id': self.viewer.selected_id,
                                                    'item_id': data_id,
-                                                   'visible': True,
+                                                   'visible': visible,
                                                    'replace': True})
 
+        # update overwrite warnings, etc
+        self._on_label_changed()
+
 
 class AddResultsMixin(VuetifyTemplate, HubListener):
     """
