@@ -212,13 +212,8 @@ class _BaseSidebarShortcut(Tool):
     viewer_attr = 'viewer'
 
     def activate(self):
-        jdaviz_state = self.viewer.jdaviz_app.state
-        jdaviz_state.drawer = True
-        tray_item_names = [tray_item['name'] for tray_item in jdaviz_state.tray_items]
-        index = tray_item_names.index(self.plugin_name)
-        if index not in jdaviz_state.tray_items_open:
-            jdaviz_state.tray_items_open = jdaviz_state.tray_items_open + [index]
         plugin = self.viewer.jdaviz_app.get_tray_item_from_name(self.plugin_name)
+        plugin.open_in_tray()
         viewer_id = self.viewer.reference_id
         viewer_select = getattr(plugin, self.viewer_attr)
         if viewer_select.multiselect:
